# Editor.py
# Francois Plamondon
# Summer 2003


import Graphics
import Tkinter
import Colors
import Tools
import MainHandler
import GFVisitors
import SaveGFVisitor
import pickle
import tkFileDialog
import sys
import string

class Workspace:
    def __init__(self, masterFrame, width, height, canvasWidth, canvasHeight, eventHandler, zoom):
        #frame containing the canvas and the scrollbars
        self.frame = Tkinter.Frame(masterFrame)
        #width of the total workspace
        self.width = width
        #height of the total workspace
        self.height = height
        #current width of the canvas
        self.canvasWidth = canvasWidth
        #current height of the canvas
        self.canvasHeight = canvasHeight
        #event handler to send events to. This is a reference to the main event handler.
        self.eventHandler = eventHandler
        #initial zoom
        self.zoom = float(zoom)
        #horizontal scrollbar
        self.xScrollbar = Tkinter.Scrollbar(self.frame, orient=Tkinter.HORIZONTAL)
        self.xScrollbar.pack(side=Tkinter.BOTTOM, fill=Tkinter.X)
        #vertical scrollbar
        self.yScrollbar = Tkinter.Scrollbar(self.frame, orient=Tkinter.VERTICAL)
        self.yScrollbar.pack(side=Tkinter.RIGHT, fill=Tkinter.Y)
        #canvas
        self.bd = 2 # borderwidth
        self.canvas = Tkinter.Canvas(self.frame, height=self.canvasHeight, width=self.canvasWidth, bg="white", borderwidth=self.bd, relief=Tkinter.SUNKEN, yscrollcommand=self.setYScrollbar, xscrollcommand=self.setXScrollbar, scrollregion=(0,0, width*self.zoom, height*self.zoom))
        #bind canvas events
        self.canvas.bind("<Button-1>", self.onButton)
        self.canvas.bind("<Button-2>", self.onButton)
        self.canvas.bind("<Button-3>", self.onButton)
        self.canvas.bind("<Double-Button-1>", self.onDoubleButton)
        self.canvas.bind("<Double-Button-2>", self.onDoubleButton)
        self.canvas.bind("<Double-Button-3>", self.onDoubleButton)
        self.canvas.bind("<Motion>", self.onMotion)
        self.canvas.bind("<Shift-Motion>", self.onShiftMotion)
        self.canvas.bind("<B1-Motion>", self.onButtonMotion)
        self.canvas.bind("<B2-Motion>", self.onButtonMotion)
        self.canvas.bind("<B3-Motion>", self.onButtonMotion)
        self.canvas.bind("<Shift-B1-Motion>", self.onShiftButtonMotion)
        self.canvas.bind("<Shift-B2-Motion>", self.onShiftButtonMotion)
        self.canvas.bind("<Shift-B3-Motion>", self.onShiftButtonMotion)
        self.canvas.bind("<ButtonRelease-1>", self.onButtonRelease)
        self.canvas.bind("<ButtonRelease-2>", self.onButtonRelease)
        self.canvas.bind("<ButtonRelease-3>", self.onButtonRelease)
        #make the canvas resizable
        self.canvas.pack(side=Tkinter.RIGHT, fill=Tkinter.BOTH, expand=1)
        self.canvas.focus_set() # focus on canvas
        self.yScrollbar.config(command=self.canvas.yview)
        self.xScrollbar.config(command=self.canvas.xview)
        self.x0 = 0
        self.y0 = 0
        #make the workspace resizable
        self.frame.pack(side=Tkinter.RIGHT, fill=Tkinter.BOTH, expand=1)

    # set the scrollbar and update y0 at the same time so that it doesn't need to
    # be updated each time an event occurs.
    def setXScrollbar(self, *args):
        apply(self.xScrollbar.set, args)
        low, hi = self.xScrollbar.get()
        #Because of the borderwidth, need to adjust the position to reflect the
        # real position of the cursor for both x0 and y0.
        self.x0 = low * self.width * self.zoom - self.bd - 1
        self.canvasWidth = (hi - low) * self.width * self.zoom

    # set the scrollbar and update y0
    def setYScrollbar(self, *args):
        apply(self.yScrollbar.set, args)
        low, hi = self.yScrollbar.get()
        self.y0 = low * self.height * self.zoom  - self.bd - 1
        self.canvasHeight = (hi - low) * self.height * self.zoom

    def getCanvas(self):
        return self.canvas

    def getCanvasWidth(self):
        return self.canvasWidth

    def getCanvasHeight(self):
        return self.canvasHeight

    def getZoom(self):
        return self.zoom

    def setZoom(self, zoom, x, y):
        offsetX = x/(self.width * self.zoom) #offsetX = relative position of cursor on workspace
        offsetY = y/(self.height * self.zoom)
        self.zoom = float(zoom)
        # change zoom, i.e. the size of the workspace
        self.canvas.config(scrollregion=(0,0, self.width * self.zoom, self.height * self.zoom))
        # calculate the percentage of the workspace occupied by half of the canvas
        halfCanvasX = (self.canvasWidth/(self.width * self.zoom))/2
        halfCanvasY = (self.canvasHeight/(self.height * self.zoom))/2
        #the offset for the upper left corner is the offset of the center minus halfCanvas
        offsetX = offsetX - halfCanvasX
        offsetY = offsetY - halfCanvasY
        self.canvas.xview(Tkinter.MOVETO, offsetX)
        self.canvas.yview(Tkinter.MOVETO, offsetY)


    def onButton(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasButton(event)


    def onDoubleButton(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasDoubleButton(event)


    def onMotion(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasMotion(event)


    def onShiftMotion(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasShiftMotion(event)


    def onButtonMotion(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasButtonMotion(event)


    def onShiftButtonMotion(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasShiftButtonMotion(event)


    def onButtonRelease(self, event):
        event.x += self.x0
        event.y += self.y0
        self.eventHandler.onCanvasButtonRelease(event)



class Editor:
    def __init__(self, root):
        self.root = root
        self.mainHandler = MainHandler.MainHandler(self)
        root.bind("<Key>", self.mainHandler.onKey)
        root.bind("<Shift-Key>", self.mainHandler.onShiftKey)
        root.bind("<Control-Key>", self.mainHandler.onControlKey)
        zoom = 1.0
        self.menuBar = Tools.MenuBar(root, self, self.mainHandler) #goes to top by itself
        self.statusBar = Tools.StatusBar(root, "", zoom, 0, 0) #goes to bottom by itself

        self.colorSelector = Colors.ColorSelector(root, self.mainHandler) #goes to bottom by itself

        self.toolFrame = Tkinter.Frame(root, relief=Tkinter.RAISED, bd=1)
        self.toolSelector = Tools.ToolSelector(self.toolFrame, self.mainHandler, Tkinter.TOP)
        self.outlineFillOptionSelector = Tools.OutlineFillOptionSelector(self.toolFrame, self.mainHandler, Tkinter.TOP)
        self.lineWidthSelector = Tools.LineWidthSelector(self.toolFrame, self.mainHandler, Tkinter.TOP)
        self.toolFrame.pack(side=Tkinter.LEFT)
        self.workspace = Workspace(root, 3400, 3400, 800, 800, self.mainHandler, zoom) #goes to the right by itself
        self.workspace.setZoom(zoom, 0, 0)
        self.canvas = self.workspace.getCanvas()
        self.GFs = self.open()
        #self.GFs = []

        self.clipboardList = []
        self.undoStack = []
        self.mainHandler.start()
        self.compositionVisitor = GFVisitors.CompositionVisitor(self)
        self.colorVisitor = GFVisitors.ColorVisitor()
        self.widthVisitor = GFVisitors.WidthVisitor()
        self.optionVisitor = GFVisitors.OptionVisitor(self)
	self.exportVisitor = SaveGFVisitor.generate_pyCode_Visitor(self)

    def getRoot(self):
        """get a reference to the root window of the editor"""
        return self.root

    def save(self):
        #zoom
        gfList = self.getGFs()
        gfListCopy = []
        for gf in gfList:
            gfListCopy.append(gf.copy())
        for gf in gfListCopy:
            gf.setEventHandler(None)
            gf.setCanvas(None)
            gf.setZoom(1.0)
        file=open("gfList.gfl", "w")
        pickle.dump(gfListCopy, file)
        file.close()

    def save1(self):
    	print "save 1"
	print	"LeafGF.OBJECT_NUMBER="+str(Graphics.LeafGF.OBJECT_NUMBER)
	gfListSorted = SaveGFVisitor.TextSortVisitor().sortGraphicalForms(self.getGFs())
	
	self.exportVisitor.gen_pyCode(gfListSorted)



    def export(self):
        filename = tkFileDialog.asksaveasfilename(title="Export",filetypes=[("Postscript","*.ps")])
        if filename != "":
            self.canvas.postscript(file=filename)

            

    def open1(self):
    	print "open1"
	
	canvas = self.canvas

        fullFileName = tkFileDialog.askopenfilename(title="open file", filetypes = [("All files", "*"), ("python file","*.py"  ) ])

# ----------------------get the filename without extention --------------------#
        name = fullFileName
        
        index = string.find(name, '/', 0)
        print index

        index1 = string.find(name, '.',0)
        print index1

        while index < index1 and index <> -1:
            name = name[index+1:]
            index = string.find(name, '/', 0)

        index2 = string.find(name, '.')
        name = name[0:index2]
        print "open1 filename: "+name
# ------------------------------------------------------------------------------
        
	
	#if "xia_object"  in sys.modules.keys():
	    #del sys.modules["xia_object"]

       

        if name in sys.modules.keys():
            del sys.modules[name]
	    
	
	try:
 	    #exec "import xia_object\n"
            #print "import xia_object\n"
            exec "import "+name+"\n"
            print "import "+name+"\n"
	except SyntaxError:							                                          # class Name not valid 
            print "Could not open graphical file"
            return []
        except IOError:                                                           # could not open file (?)
            print "Could not open graphical file"
            return []
        except ImportError:                                                       # could not open file...
            print "Could not open graphical file"
            return []
	    
	#className = eval('xia_object.xia_object')
        className = eval(name+'.'+name)
	new_obj = className(self.canvas)
	print new_obj.gFs
	print new_obj
	new_obj.DrawObject(self)
            
    def open(self):
        #return  []
        f = open("gfList.gfl", "r")
        gfList = pickle.load(f)
        for gf in gfList:
            gf.setCanvas(self.canvas)
            gf.setEventHandler(self.mainHandler)
        return gfList
#        return [Graphics.Rectangle(50, 50, 90, 90, canvas=self.canvas, outline="black", fill="blue", width=2, eventHandler=self.mainHandler),
#                    Graphics.Oval(50, 50, 90, 90, canvas=self.canvas, outline="black", fill="green", eventHandler=self.mainHandler),
#                    Graphics.Text(70, 90, canvas=self.canvas, zoom=zoom, eventHandler=self.mainHandler, text="text", fill="red"),
#                    Graphics.Connector(100, 100, canvas=self.canvas, zoom=zoom, eventHandler=self.mainHandler),
#                    Graphics.Polygon([50, 50, 90, 90, 32, 2], canvas=self.canvas,  outline="green", fill="purple", width=2, eventHandler=self.mainHandler),
#                    Graphics.Line([50, 10, 50, 50, 132, 50], canvas=self.canvas, fill="black", eventHandler=self.mainHandler),
#                    Graphics.Composite([Graphics.Rectangle(50, 20, 140, 100, canvas=self.canvas, outline="black", fill="yellow", width=2, eventHandler=self.mainHandler),
#                                          Graphics.Rectangle(50, 50, 100, 120, canvas=self.canvas, outline="black", fill="purple", width=2, eventHandler=self.mainHandler),
#                                          Graphics.Oval(70, 20, 40, 90, canvas=self.canvas, outline="black", fill="green", width=2, eventHandler=self.mainHandler),
#                                          Graphics.Oval(50, 50, 40, 20, canvas=self.canvas, outline="black", fill="gray", width=2, eventHandler=self.mainHandler)], canvas=self.canvas, zoom=zoom, eventHandler=self.mainHandler)]


    def cut(self, gfList):
        if len(gfList) > 0:
            previousClipboard = self.clipboardList
            self.clipboardList = gfList
            indexList = []
            for gf in gfList:
                gf.setCanvas(None)
                indexList.append(self.removeGF(gf))
            pairList = map(None, gfList, indexList)
            self.undoStack.append((self.undo_cut, [previousClipboard, pairList]))


    def copy(self, gfList):
        if len(gfList) > 0:
            previousClipboard = self.clipboardList
            self.clipboardList = []
            for gf in gfList:
                cgf = gf.copy()
                self.clipboardList.append(cgf)
                cgf.setCanvas(None)


    def paste(self):
        clipboardCopy = []
        for gf in self.clipboardList:
            c = gf.copy()
            c.setZoom(self.getZoom())
            clipboardCopy.append(c)
        for gf in clipboardCopy:
            gf.translate(10, 10)
            gf.setCanvas(self.canvas)
            self.appendGF(gf)
        self.undoStack.append((self.undo_paste, [clipboardCopy]))
        return clipboardCopy

    #The GFs in gfList are not necessarily in the order in which they are drawn.
    #Since we want the relative order of the selection to be preserved, we first bring
    #to top the GF which is drawn first.
    def bringToTop(self, gfList):
        allGFs = self.getGFs()
        pairList = []
        for gf in allGFs:
            if gf in gfList:
                pairList.append((gf, self.bringToTopGF(gf)))
        if len(pairList) > 0:
            self.undoStack.append((self.undo_bringPush, [pairList]))

    #(see bringToTop) Here we need to reverse the list to push the GFs in the right order.
    def pushToBottom(self, gfList):
        allGFs = self.getGFs()
        allGFs.reverse()
        pairList = []
        for gf in allGFs:
            if gf in gfList:
                pairList.append((gf, self.pushToBottomGF(gf)))
        if len(pairList) > 0:
            self.undoStack.append((self.undo_bringPush, [pairList]))

    def compose(self, gfList):
        if len(gfList) > 1: # create a composite only if it's worth it
            allGFs = self.getGFs()
            sortedList = []
            for gf in allGFs:
                if gf in gfList:
                    sortedList.append(gf)
            indexList = []
            for gf in sortedList:
                indexList.append(self.removeGF(gf))
            composite = Graphics.Composite(sortedList, canvas=self.canvas, zoom=self.getZoom(), eventHandler=self.mainHandler)
            self.insertGF(indexList[len(indexList)-1], composite)
            self.undoStack.append((self.undo_compose, [composite, indexList]))
            return [composite]
        else:
            return gfList

    def decompose(self, gfList):
        compositeList = []
        for gf in gfList:
            componentList = self.compositionVisitor.decompose(gf)
            if len(componentList) > 0: #if gf was decomposed
                compositeList.append(gf) #add the composite to the list of composites
        self.undoStack.append((self.undo_decompose, [compositeList]))

    def delete(self, gfList):
        indexList = []
        for gf in gfList:
            gf.setCanvas(None)
            indexList.append(self.removeGF(gf))
        pairList = map(None, gfList, indexList)
        self.undoStack.append((self.undo_delete, [pairList]))

    def getBoundingBox(self, gfList):
        boxes = []
        for gf in gfList:
            if gf.getCanvas() != None:
                boxes.append(gf.getApproxBoundingBox())
        if len(boxes) == 0:
            raise TypeError, "gfList contains no active GF"
        xMin = boxes[0][0]
        yMin = boxes[0][1]
        xMax = boxes[0][2]
        yMax = boxes[0][3]
        for box in boxes:
            if box[0] < xMin:
                xMin = box[0]
            if box[1] < yMin:
                yMin = box[1]
            if box[2] > xMax:
                xMax = box[2]
            if box[3] > yMax:
                yMax = box[3]
        return (xMin, yMin, xMax, yMax)

    def setFillColor(self, gfList, color):
        undoList = self.colorVisitor.setFillColor(gfList, color)
        if len(undoList) > 0:
            self.undoStack.append((self.undo_setFillColor, [undoList]))

    def setOutlineColor(self, gfList, color):
        undoList = self.colorVisitor.setOutlineColor(gfList, color)
        if len(undoList) > 0:
            self.undoStack.append((self.undo_setOutlineColor, [undoList]))

    def setLineWidth(self, gfList, lineWidth):
        undoList = self.widthVisitor.setWidth(gfList, lineWidth)
        if len(undoList) > 0:
            self.undoStack.append((self.undo_setLineWidth, [undoList]))

    def setOutlineFillOption(self, gfList, option):
        undoList = self.optionVisitor.changeOption(gfList, option)
        if len(undoList) > 0:
            self.undoStack.append((self.undo_setOutlineFillOption, [undoList]))

    def createRectangle(self, xy):
        gf = Graphics.Rectangle(xy[0], xy[1], xy[2], xy[3], canvas=self.canvas, outline=self.getOutlineColor(), outlineOption=self.hasOutline(), fill=self.getFillColor(), fillOption=self.hasFill(), width=self.getLineWidth(), zoom=self.getZoom(), eventHandler=self.mainHandler)
	gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createOval(self, xy):
        gf = Graphics.Oval(xy[0], xy[1], xy[2], xy[3], canvas=self.canvas, outline=self.getOutlineColor(), outlineOption=self.hasOutline(), fill=self.getFillColor(), fillOption=self.hasFill(), width=self.getLineWidth(), zoom=self.getZoom(), eventHandler=self.mainHandler)
	gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createLine(self, xy, smooth=0):
        gf = Graphics.Line(xy, canvas=self.canvas, fill=self.getFillColor(), width=self.getLineWidth(), zoom=self.getZoom(), eventHandler=self.mainHandler, smooth = smooth)
	gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createPolygon(self, xy, smooth=0):
        gf = Graphics.Polygon(xy, canvas=self.canvas, outline=self.getOutlineColor(), outlineOption=self.hasOutline(), fill=self.getFillColor(), fillOption=self.hasFill(), width=self.getLineWidth(), zoom=self.getZoom(), eventHandler=self.mainHandler, smooth = smooth)
	gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createConnector(self, xy):
        gf = Graphics.Connector(xy[0], xy[1], canvas=self.canvas, zoom=self.getZoom(), eventHandler=self.mainHandler)
        gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createImage(self, xy, filename):
        gf = Graphics.Image(xy[0], xy[1], filename, canvas=self.canvas, zoom=self.getZoom(), eventHandler=self.mainHandler)
        gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf

    def createText(self, xy, text):
        gf = Graphics.Text(xy[0], xy[1], canvas=self.canvas, zoom=self.getZoom(), eventHandler=self.mainHandler, text=text, fill=self.getFillColor())
        gf.createItem()
        self.appendGF(gf)
        self.undoStack.append((self.undo_create, [[gf]]))
        return gf


    def isUndoStackEmpty(self):
        if len(self.undoStack) == 0:
            return 1
        else:
            return 0

    def isClipboardEmpty(self):
        if len(self.clipboardList) == 0:
            return 1
        else:
            return 0


    def translate(self, gfList, dx, dy):
        for gf in gfList:
            gf.translate(dx, dy)
        self.undoStack.append((self.undo_translate, [gfList, -dx,-dy]))

    def rotate(self, gfList, centerX, centerY, angle):
        for gf in gfList:
            gf.rotate(centerX, centerY, angle)
        self.undoStack.append((self.undo_rotate, [gfList, centerX, centerY, -angle]))

    def scale(self, gfList, centerX, centerY, factorX, factorY):
        for gf in gfList:
            gf.scale(centerX, centerY, factorX, factorY)
        self.undoStack.append((self.undo_scale, [gfList, centerX, centerY, 1/factorX, 1/factorY]))

# the following method is used by the translation handler to add an undo command.
# Instead of creating an undo for each small translation (by using the normal translate method of the editor),
# the handler calls the "add undo translation" method of the editor when the translation is complete.
    def addUndoTranslate(self, gfList, dx, dy):
        if dx != 0 and dy != 0:
            self.undoStack.append((self.undo_translate, [gfList, -dx,-dy]))

# see addUndoTranslate
    def addUndoRotate(self, gfList, centerX, centerY, angle):
        self.undoStack.append((self.undo_rotate, [gfList, centerX, centerY, -angle]))

# see addUndoTranslate
    def addUndoScale(self, gfList, centerX, centerY, factorX, factorY):
        self.undoStack.append((self.undo_scale, [gfList, centerX, centerY, 1/factorX, 1/factorY]))

    def addUndoSetCoords(self, gf, oldCoords):
        self.undoStack.append((self.undo_setCoords, [gf, oldCoords]))

    def undo(self):
        method, args = self.undoStack.pop()
        apply(method, args)


#######################
# undo commands
    def undo_create(self, gfList):
        for gf in gfList:
            gf.setCanvas(None)
            self.removeGF(gf)


    def undo_delete(self, pairList):
        # pairList is a list of pairs, where each
        # pair contains an inactive GF and its position in the canvas.
        # pairList must be reversed because the index is valid
        # for the editor's list after the other gfs are deleted.
        pairList.reverse()
        for gf, index in pairList:
            gf.setCanvas(self.canvas)
            self.insertGF(index, gf)

    def undo_cut(self, previousClipboard, pairList):
        # pairList is a list of pairs, where each
        # pair contains an inactive GF and its position in the canvas.
        # pairList must be reversed because the index is valid
        # for the editor's list after the other gfs are deleted.
        pairList.reverse()
        for gf, index in pairList:
            gf.setCanvas(self.canvas)
            self.insertGF(index, gf)
        self.clipboardList = previousClipboard

    def undo_copy(self, previousClipboard):
        self.clipboardList = previousClipboard
       
    def undo_paste(self, pastedList):
        for gf in pastedList:
            gf.setCanvas(None)
            self.removeGF(gf)

    def undo_bringPush(self, pairList):
        pairList.reverse()
        for gf, index in pairList:
            self.removeGF(gf)
            self.insertGF(index, gf)

    def undo_compose(self, compositeGF, indexList):
        gfList = compositeGF.getComponents()
        self.removeGF(compositeGF)
        pairList = map(None, gfList, indexList)
        pairList.reverse()
        for gf, index in pairList:
            gf.setEventHandler(self.mainHandler)
            self.insertGF(index, gf)

    def undo_decompose(self, compositeList):
        for composite in compositeList:
            gfList = composite.getComponents()
            for gf in gfList:
                index = self.removeGF(gf)
                gf.setEventHandler(composite)
            self.insertGF(index, composite)

    def undo_translate(self, gfList, dx, dy):
        for gf in gfList:
            gf.translate(dx, dy)

    def undo_rotate(self, gfList, centerX, centerY, angle):
        for gf in gfList:
            gf.rotate(centerX, centerY, angle)

    def undo_scale(self, gfList, centerX, centerY, factorX, factorY):
        for gf in gfList:
            gf.scale(centerX, centerY, factorX, factorY)

    def undo_setCoords(self, gf, oldCoords):
        gf.setCoords(oldCoords)

    def undo_setFillColor(self, undoList):
        for gf, color in undoList:
            gf.setFillColor(color)

    def undo_setOutlineColor(self, undoList):
        for gf, color in undoList:
            gf.setOutlineColor(color)

    def undo_setLineWidth(self, undoList):
        for gf, width in undoList:
            gf.setWidth(width)

    def undo_setOutlineFillOption(self, undoList):
        for gf, (outlineOption, fillOption) in undoList:
            gf.setOutlineOption(outlineOption)
            gf.setFillOption(fillOption)

######################
    def appendGF(self, gf):
        self.GFs.append(gf)

    def insertGF(self, index, gf):
        self.GFs.insert(index, gf)
        for gf in self.GFs:
            gf.bringToTop()

    def getIndex(self, gf):
        return self.GFs.index(gf)

    def removeGF(self, gf):
        index = self.GFs.index(gf)
        self.GFs.remove(gf)
        return index #return the index the gf had in the list (which was also its relative position on the canvas)

    # convention: the first gfs in the list are drawn first.
    # The order in which the GFs are drawn is useful when we want to save.
    def bringToTopGF(self, gf):
        gf.bringToTop()
        index = self.removeGF(gf)
        self.GFs.append(gf)
        return index

    def pushToBottomGF(self, gf):
        gf.pushToBottom()
        index = self.removeGF(gf)
        self.GFs.insert(0, gf)
        return index


    def getGFs(self):
        return self.GFs[:]

    def getEventHandler(self):
        return self.mainHandler

    def getCanvasWidth(self):
        return self.workspace.getCanvasWidth()

    def getCanvasHeight(self):
        return self.workspace.getCanvasHeight()

    def getCanvas(self):
        return self.workspace.getCanvas()

    def getZoom(self):
        return self.workspace.getZoom()

    def setZoom(self, zoom, x, y):
        self.workspace.setZoom(zoom, x, y)
        # update statusBar and GFs to the new zoom (whether it changed or not)
        newZoom = self.workspace.getZoom()
        self.statusBar.setZoom(newZoom)
        for gf in self.GFs:
            gf.setZoom(newZoom)
    
    def setToolSelector(self, tool):
        self.toolSelector.set(tool)

    def getLineWidth(self):
        return self.lineWidthSelector.get()

    def getOutlineColor(self):
        return self.colorSelector.getOutlineColor()

    def getFillColor(self):
        return self.colorSelector.getFillColor()

    def hasOutline(self):
        option = self.outlineFillOptionSelector.get()
        return option[0]

    def hasFill(self):
        option = self.outlineFillOptionSelector.get()
        return option[1]


####################


if __name__ == "__main__":  
    root = Tkinter.Tk()
    root.title("Icon Editor - AToM3")
    root.geometry("%dx%d%+d%+d" % (800, 800, 250, 50))
    editor = Editor(root)
    root.mainloop()




