import Graphics
import GFVisitors
import string
from GFVisitors import GFVisitor
import tkFileDialog

class TextSortVisitor( GFVisitor ):
  """ 
  Sorts the graphical forms so that text is placed at the end 
  This is important as we want to be able to read the text, not have it buried
  under a heavy pile of graph forms :D
  """
  
  def __init__(self):
      pass
    
  def sortGraphicalForms( self, gfList ):    
    """ Returns the gfList with text items at the end """
    self.regularItemsList = []
    self.textItemsList = []
    
    for gf in gfList:
        gf.accept(self)
        
    return self.regularItemsList + self.textItemsList 
    
  def visitRectangle(self, gf):
      self.regularItemsList.append( gf )
        
  def visitOval(self, gf):
      self.regularItemsList.append( gf )
      
  def visitPolygon(self, gf):
      self.regularItemsList.append( gf )
  
  def visitLine(self, gf):
      self.regularItemsList.append( gf )
  
  def visitText(self, gf):
      self.textItemsList.append( gf )

  def visitImage(self, gf):
      self.regularItemsList.append( gf )
  
  def visitConnector(self, gf):
      self.regularItemsList.append( gf )
  
  def visitComposite(self, gf):
      list = gf.getComponents()
      for gf in list:
        gf.accept(self)
    

class generate_pyCode_Visitor( GFVisitor ):
  
  def __init__(self, editor ):
     # self.className      = editor.className
      self.editor         = editor

  
    
  def gen_pyCode(self, gfList ):
       
    
      #fileName = os.path.join( self.modelPath, name + '.py' )
     
      name =tkFileDialog.asksaveasfilename(title = 'save') 
      
      #name = "xia_object"
      #fileName = "xia_object.py"
      print "will generate " + name
     
      f = open( name, "w+t" )            
      if( f == None): return None
      
      # Check if we actually have a drawing
      self.bbox = self.editor.getBoundingBox( self.editor.getGFs() )
      self.size = ( self.bbox[2] - self.bbox[0], self.bbox[3] - self.bbox[1] )
      if( self.size[0] == 0 and self.size[1] == 0):                         
          f.close()
          try:
            os.remove(fileName)
            os.remove(fileName+'c')     # can fail!
          except OSError:
            pass
          return None
        
     
      classIndent = '    '
      methodIndent = classIndent + classIndent
      
# ------------------- get file name with extention and its directory -----------           
      index = string.find(name, '/', 0)
      print index

   
      while index <> -1:
        name = name[index+1:]
        index = string.find(name, '/', 0)

      index2 = string.find(name, '.')
      if index2 <> -1:
        name = name[0:index2]
      print "save1 filename [no extention ]: "+ name

# --------------------generate header code----------------------------------
      
      self.generateHeaderCode( f, name, classIndent, methodIndent )
#-----------------------------------------------------------------------------


      # Generate code for each graphical form
      self.f = f
      self.CI = classIndent
      self.MI = methodIndent     
                  
      for gf in gfList:
        gf.accept(self)
	print gf.type
#	print gf.fill
#	print 'gf.stripple = '+gf.stipple

      self.generateTailCode( f )
        
      f.close()


      

  def visitRectangle(self, gf ):
      """ Exports a rectangular object """
      self.visitRegularObject( 'Rectangle', gf )
        
  def visitOval(self, gf):
      """ Exports a circle/oval object """
      self.visitRegularObject( 'Oval', gf )

  def visitLine(self, gf):
      self.visitRegularObject( 'Line', gf )

  def visitPolygon(self, gf):
      self.visitRegularObject( 'Polygon', gf)

  def visitText(self, gf):
      self.visitRegularObject( 'Text', gf )

  def visitImage(self, gf):
      self.visitRegularObject( 'Image', gf )

  def visitConnector(self, gf):
      self.visitRegularObject( 'Connector', gf)
  
  def visitComposite(self, gf):
      list = gf.getComponents()
      for gf in list:
        gf.accept(self)
  
      
  def visitRegularObject( self, objectType, gf ):
      """ Generates the graphical code for common objects """
    
      f=self.f
      indent = self.MI
      gfName = 'gf'+str(gf.getObjectNumber())
      gfType = gf.type
      
      f.write(indent + '\n')

# ---------------------- All ------------------------------

      f.write(indent + 'xy = '+ str(gf.getCoords()) + '\n')
      f.write(indent + 'isLock = '+str(gf.getLock()) + '\n')
      f.write(indent + 'type = "'+str(gf.type) + '"\n')


# --------------------- Connector ----------------------

      if gfType == 'Connector':
        f.write(indent + 'x = xy[0]\n')
        f.write(indent + 'y = xy[1]\n')
        f.write(indent + 'zoom = '+ str(gf.getZoom()) + '\n')

        f.write(indent + gfName + '= Graphics.'+ gfType + '(x=x, y=y, canvas=canvas,  zoom=zoom, locked = isLock ) \n')



# --------------------- Image -------------------------

      if gfType == 'Image':
        f.write(indent + 'x = xy[0]\n')
        f.write(indent + 'y = xy[1]\n')
        f.write(indent + 'filename = "'+str(gf.getFilename()) + '"\n')
        f.write(indent + 'zoom = '+ str(gf.getZoom()) + '\n')
        f.write(indent + 'anchor = "'+str(gf.getAnchor()) + '"\n')

        f.write(indent + gfName + '= Graphics.'+ gfType + '(x=x, y=y, filename=filename, canvas=canvas,  zoom=zoom, locked = isLock, anchor=anchor ) \n')
        
  



# ---------------------- Text --------------------------


      if gfType == 'Text':
        f.write(indent + 'x = xy[0]\n')
        f.write(indent + 'y = xy[1]\n')
        f.write(indent + 'family = "'+str(gf.getFamily()) + '"\n')      
        f.write(indent + 'text = "'+str(gf.getText()) + '"\n')      
        f.write(indent + 'size = '+str(gf.getSize()) + '\n')      
        f.write(indent + 'bold = '+str(gf.getBold()) + '\n')      
        f.write(indent + 'italic = '+str(gf.getItalic()) + '\n')    
        f.write(indent + 'underline = '+str(gf.getUnderline()) + '\n') 
        f.write(indent + 'anchor ="'+str(gf.getAnchor()) + '"\n')
        f.write(indent + 'fill = "'+str(gf.getFillColor()) + '"\n') 

        f.write(indent + gfName + '= Graphics.'+ gfType + '(x=x, y=y, canvas=canvas,  text = text, fill=fill, locked = isLock, family=family, size=size, bold=bold, italic=italic, underline=underline, anchor=anchor ) \n')
        
        
        


# ------------------- Line / Rectangle / Oval / Polygone ----------------
        
      if gfType == 'Rectangle' or gfType == 'Oval' or gfType == 'Polygon' or gfType == 'Line':
      
        f.write(indent + 'width = '+str(gf.getWidth()) + '\n')
        f.write(indent + 'fill = "'+str(gf.getFillColor()) + '"\n')
        stipple = gf.getStipple()
        if stipple == "":      
          f.write(indent + 'stipple = ""\n')
        else:
          f.write(indent + 'stipple = '+str(gf.getStipple()) + '\n')  
        f.write(indent + 'zoom = '+ str(gf.getZoom()) + '\n')
      
#-----------------------  Line ----------------------------------------
      if gfType == 'Line':
        f.write(indent + 'arrow = "'+str(gf.getArrow()) + '"\n')
        f.write(indent + 'capStyle = "'+str(gf.getCapstyle()) + '"\n')
        f.write(indent + 'joinStyle = "'+str(gf.getJoinstyle()) + '"\n')
        f.write(indent + 'arrow = "'+str(gf.getArrow()) + '"\n')
        f.write(indent + 'smooth = "'+str(gf.getSmooth()) + '"\n')
        stipple = gf.getStipple()  

        f.write(indent + gfName + '= Graphics.'+ gfType + '(xy=xy, canvas=canvas,  fill=fill,  width=width, stipple=stipple, zoom=zoom, locked = isLock, arrow = arrow, capstyle=capStyle, joinstyle=joinStyle, smooth = smooth ) \n')
        
      
        
#---------------------- Rectangle / Oval / Polygon -------------------------
        
      if gfType == 'Rectangle' or gfType == 'Oval' or gfType == 'Polygon':
        f.write(indent + '\n')
        f.write(indent + 'x0 = xy[0] \n')
        f.write(indent + 'y0 = xy[1] \n')
        f.write(indent + 'x1 = xy[2] \n')
        f.write(indent + 'y1 = xy[3] \n')      

        f.write(indent + 'outline = "'+str(gf.getOutlineColor()) + '"\n')
        f.write(indent + 'fillOption = '+str(gf.getFillOption()) + '\n')
        f.write(indent + 'outlineOption = '+str(gf.getOutlineOption()) + '\n')
        stipple = gf.getStipple()

# ------------------------------- Rectangle / Oval ------------------------------

      if gfType == 'Rectangle' or gfType == 'Oval':       

        f.write(indent + gfName + '= Graphics.'+ gfType + '(x0=x0, y0=y0, x1=x1, y1=y1, canvas=canvas, outline=outline, outlineOption=outlineOption, fill=fill, fillOption=fillOption, width=width, stipple=stipple, zoom=zoom, locked = isLock ) \n')

# --------------------------------- Polygon -----------------------------------
        
      if gfType == 'Polygon':
        f.write(indent + 'smooth = "'+str(gf.getSmooth()) + '"\n')
        
      if gfType == 'Polygon':       

        f.write(indent + gfName + '= Graphics.'+ gfType + '(xy=xy, canvas=canvas, outline=outline, outlineOption=outlineOption, fill=fill, fillOption=fillOption, width=width, stipple=stipple, zoom=zoom, locked = isLock, smooth = smooth ) \n')
        

# --------------------- Composite ----------------------
      #if gfType == 'Composite':
      #  f.write(indent + 'GFs = '+ str(gf.getComposites())+'\n')        
       # f.write(indent + 'zoom = '+ str(gf.getZoom()) + '\n')
        #f.write(indent + 'indexList = '+ str(gf.getIndexList())+'\n')
        #f.write(indent + gfName + '= Graphics.'+ gfType + '(GFs=GFs, indexList=indexList, canvas=canvas, zoom=zoom, locked = isLock) \n')      
        #f.write(indent + 'for gf in GFs:\n')
        #f.write(indent + '  gf.accept(self) \n') 
        #f.write(indent + 'editor.insertGF(indexList[len(indexList)-1],'+gfName+')\n')
        #f.write(indent + 'editor.undoStack.append((self.undo_compose, ['+gfName+', indexList]))\n')
        


#----------------------- All --------------------------------------------
     # if gf.getIsleaf()=='true':
      
      f.write(indent + 'self.gFs.append( '+gfName+' ) \n\n')


             
      

      
  def generateHeaderCode(self, f, name, classIndent, methodIndent  ):
      
     
      f.write('import Graphics\n')
      f.write('import Tkinter\n')
      f.write('class ' + name + ':\n\n')      
      f.write(classIndent+'def __init__(self,canvas):\n')        
      
      # A list of GraphicalForms
      f.write(methodIndent+'self.gFs = []\n\n')	
      f.write(methodIndent+ 'self.eventhandler = None\n\n')

      
  def generateTailCode(self, f):

      # The beginning of the drawing specification
      f.write('\n')
      f.write(self.CI+'def DrawObject(self, editor):\n')
      f.write(self.MI + 'for gf in self.gFs:\n')
      f.write(self.MI + '  gf.setEventHandler(editor.mainHandler)\n\n')
      f.write(self.MI + '  gf.createItem()\n')
      f.write(self.MI + '  editor.appendGF(gf)\n')
      f.write(self.MI + '  editor.undoStack.append((editor.undo_create, [[gf]]))\n')

      f.write('\n')
      f.write(self.CI+'def showObj(self):\n')
      f.write(self.MI + 'for gf in self.gFs:\n')
      f.write(self.MI + '  gf.createItem()\n')

      f.write('\n')
      f.write(self.CI+'def hideObj(self):\n')
      f.write(self.MI + 'for gf in self.gFs:\n')
      f.write(self.MI + '  gf.deleteItem()\n')

      f.write('\n')
      f.write(self.CI+'def getCoords(self):\n')
      f.write(self.MI + 'xy_bound = self.gFs[0].xy\n')
      f.write(self.MI + 'for gf in self.gFs:\n')
      f.write(self.MI + '  if gf.xy[0] < xy_bound[0]:  \n')
      f.write(self.MI + '    xy_bound[0] = gf.xy[0]  \n')
      f.write(self.MI + '  if gf.xy[1] < xy_bound[1]:  \n')
      f.write(self.MI + '    xy_bound[1] = gf.xy[1]  \n')
      f.write(self.MI + '  if gf.xy[2] > xy_bound[2]:  \n')
      f.write(self.MI + '    xy_bound[2] = gf.xy[2]  \n')
      f.write(self.MI + '  if gf.xy[3] > xy_bound[3]:  \n')
      f.write(self.MI + '    xy_bound[3] = gf.xy[3]  \n')
      f.write(self.MI + 'self.xy = xy_bound\n')
      f.write(self.MI + 'return xy_bound \n')
      

